//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================

using System;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;

using YAC_NET.Data;
using YAC_NET.Properties;
using System.Text;

namespace YAC_NET
{
    /// <summary>
    /// The call logger logs calls to a file.
    /// 
    /// The ACTIVE log is NOT valid XML. It contains XML fragments, one for each CallerID entry.
    /// This is so that we can merely append a caller ID to this file without reading it in for 
    /// every call. Appending is very simple in comparison.
    /// 
    /// When we read this file in to display or edit it, we prepend and append the proper XML 
    /// tags to convert it to the correct XML for a CallLog object. Then we deserialize it.
    /// </summary>
    class CallLogger
    {
       
        /// <summary>
        /// Add a new callerID event to a file
        /// </summary>
        /// <param name="callerID"></param>
        internal static void LogCall(CallerIDType callerID)
        {
            Trace.TraceError("CallLogger.LogCall");
            try
            {
                RotateLog();

                using (StreamWriter sw = File.AppendText(CallLog.FileName))
                    sw.WriteLine(callerID.Serialize());
            }
            catch (Exception ex)
            {
                Trace.TraceError("CallLogger.LogCall: Error logging call:\r\n{0}", ex.ToString());
            }
        }

        /// <summary>
        /// Rotate the log file
        /// </summary>
        private static void RotateLog()
        {
            if (!TimeToRotate())
                return;

            try
            {
                CallLog oldLog = ReadLog();
                oldLog.Month = DateTime.Now.Month.ToString();
                oldLog.Year = DateTime.Now.Year.ToString();

                oldLog.Save(string.Format("CallLog{0:MM}-{1:yyyy}.xml", DateTime.Now, DateTime.Now));

                File.Delete(CallLog.FileName);
            }
            catch (Exception ex)
            {
                Trace.TraceError("CallLogger.RotateLog: Error rotating log:\r\n{0}", ex.ToString());
            }
        }

        /// <summary>
        /// Check if it's time to rotate the log file
        /// </summary>
        /// <returns></returns>
        private static bool TimeToRotate()
        {
            if (!Configuration.Current.LastLog.HasValue)
            {
                Configuration.Current.LastLog = DateTime.Now;
                return false;
            }

            DateTime lastLog = Configuration.Current.LastLog.Value;

            if (lastLog.Month - DateTime.Now.Month ==0)
                return false;

            Configuration.Current.LastLog = DateTime.Now;
            return true;

        }

        /// <summary>
        /// Read the most recent log into the data model
        /// </summary>
        /// <returns></returns>
        internal static CallLog ReadLog()
        {
            Trace.TraceError("CallLogger.ReadLog");

            try
            {
                if (!File.Exists(CallLog.FileName))
                    return new CallLog();
                else
                    return CallLog.Load();
            }
            catch (Exception ex)
            {
                Trace.TraceError("CallLogger.ReadLog: Error loading call log:\r\n{0}", ex.ToString());
            }

            return null;
        }

        /// <summary>
        /// Save the log (after possibly clearing or removing entries)
        /// </summary>
        /// <param name="m_callLog"></param>
        internal static void SaveLog(CallLog m_callLog)
        {
            Trace.TraceError("CallLogger.SaveLog");

            if (File.Exists(CallLog.FileName))
                File.Delete(CallLog.FileName);

            #region write all the callerID entries to the string
            StringWriter stringWriter = new StringWriter();
            foreach (CallerIDType callerID in m_callLog.CallerIDType)
                stringWriter.WriteLine(callerID.Serialize());

            #endregion

            #region write the string to the file
            using (StreamWriter sw = File.AppendText(CallLog.FileName))
                sw.Write(stringWriter.ToString());
            #endregion

        }
    }
}
