//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO.Ports;
using System.Diagnostics;

using YAC_NET.Data;
using YAC_NET.Properties;

namespace YAC_NET.Controls
{
    /// <summary>
    /// 
    /// </summary>
    public partial class ModemControl : UserControl
    {
        /// <summary>
        /// 
        /// </summary>
        public ModemControl()
        {
            InitializeComponent();
            m_okEvent = new AutoResetEvent(false);
            m_errorEvent = new AutoResetEvent(false);
            m_eventArray = new WaitHandle[2] { m_errorEvent, m_okEvent };
        }

        #region fields
        SerialPort m_serialPort;
        Thread m_queryThread;
        EventWaitHandle m_okEvent;
        EventWaitHandle m_errorEvent;
        WaitHandle[] m_eventArray;
        #endregion

        /// <summary>
        /// Load the form, do the databinding
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ModemControl_Load(object sender, EventArgs e)
        {
            if (this.DesignMode)
                return;

            string[] portNames = SerialPort.GetPortNames();
            if (portNames.Length==0)
            {
                MessageBox.Show(
                    this,
                    Resources.Error_ModemNoComPorts, 
                    Resources.Caption_Error, 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }
            else
            {
                portComboBox.DataSource = portNames;
                if (Configuration.Current.IsFirstRun)
                    Configuration.Current.Modem.Port = portNames[0];
                configurationBindingSource.DataSource = Configuration.Current;
            }
        }

        /// <summary>
        /// The Query Thread
        /// </summary>
        void Query()
        {
            string failureReason = Resources.Message_ModemFailureNoCallerIDString;
            try
            {
                // send each init string to the modem in turn, and wait for an OK or ERROR
                // response in the com port data callback
                foreach (string value in Settings.Default.ModemInitStrings)
                {
                    m_serialPort.Write(value + "\r");

                    // wait for a response signal
                    if (WaitHandle.WaitAny(m_eventArray) == 1)
                    {
                        BeginInvoke((MethodInvoker)delegate()
                        {
                            Configuration.Current.Modem.InitString = value;

                            MessageBox.Show(
                                this,
                                string.Format(Resources.Message_CallerIDConfigString, value),
                                Resources.Caption_Success,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Information);
                        });
                        failureReason = string.Empty;
                        return;
                    }
                }

            }
            catch (TimeoutException ex)
            {
                Trace.TraceError("Timeout writing to port:{0}\r\n{1}",m_serialPort.PortName, ex.ToString());
                failureReason = Resources.Message_ModemFailureTimeout;
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error procesing in modem thread:\r\n{0}", ex.ToString());
                failureReason = ex.Message;
            }
            finally
            {
                // if the failure reasons in null, do nothing
                if (!string.IsNullOrEmpty(failureReason))
                {
                    // if we made it to here then we failed to get a proper initialization string
                    BeginInvoke((MethodInvoker)delegate()
                    {
                        MessageBox.Show(
                            this,
                            string.Format(Resources.Error_CouldNotDiscoverCallerIDString, failureReason),
                            Resources.Caption_Failure,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                    });
                }

                if (m_serialPort.IsOpen)
                    m_serialPort.Close();

                BeginInvoke((MethodInvoker)delegate() { Cursor = Cursors.Default; });
            }

        }

        /// <summary>
        /// Event handler for data received on the com port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void m_serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // receive a value
            string value = m_serialPort.ReadExisting();

            if (this.InvokeRequired)
                BeginInvoke((MethodInvoker)delegate() { myModemTextBox.AppendText(value); });

            // check if it's an OK or a ERROR
            if (value.Contains("OK"))
            {
                // signal that we're done
                m_okEvent.Set();
                return;
            }

            if (value.Contains("ERROR"))
            {
                // signal to read more
                m_errorEvent.Set();
                return;
            }

        }

        /// <summary>
        /// Event handler for error received on the COM port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void m_serialPort_ErrorReceived(object sender, SerialErrorReceivedEventArgs e)
        {
            Trace.TraceError("Error talking to modem: {0}", e.EventType.ToString());
        }

        /// <summary>
        /// start testing the modem for init strings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myStartButton_Click(object sender, EventArgs e)
        {
            try
            {
                m_serialPort = new SerialPort(Configuration.Current.Modem.Port, 2400, Parity.None, 8, StopBits.One);
                m_serialPort.DtrEnable = true;
                m_serialPort.Handshake = Handshake.RequestToSend;
                m_serialPort.Encoding = Encoding.ASCII;
                m_serialPort.ReadTimeout = 1000;
                m_serialPort.WriteTimeout = 1000;

                m_serialPort.ErrorReceived += new SerialErrorReceivedEventHandler(m_serialPort_ErrorReceived);
                m_serialPort.DataReceived += new SerialDataReceivedEventHandler(m_serialPort_DataReceived);
                m_serialPort.Open();

                Cursor = Cursors.WaitCursor;

                m_queryThread = new Thread(Query);
                m_queryThread.Name = "Modem Configuration Thread";
                m_queryThread.Start();
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error starting modem query thread:\r\n{0}", ex.ToString());
                MessageBox.Show(
                    this,
                    string.Format(Resources.Error_ModemCommunication, Configuration.Current.Modem.Port),
                    Resources.Caption_Error, 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myCancelButton_Click(object sender, EventArgs e)
        {
            // signal the thread that we're done
            m_okEvent.Set();

            if (m_serialPort != null)
                m_serialPort.Close();

            ParentForm.Close();
        }


    }
}
