//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Collections.Specialized;

using YAC_NET.Properties;

namespace YAC_NET.Data
{

    /// <summary>
    /// The call log is a collection of callerID events that can be persisted in a file
    /// </summary>
    public partial class CallLog
    {
        private const string s_callLogHeader = "<?xml version=\"1.0\" encoding=\"utf-8\"?><CallLog xmlns=\"http://BlueToque.ca/CallerID\">";
        private const string s_callLogFooter = "</CallLog>";

        /// <summary>
        /// Default constructor
        /// </summary>
        public CallLog()
        {
            CallerIDType = new CallerIDTypeCollection();
        }

        /// <summary>
        /// Sort the call log
        /// </summary>
        internal void Sort(SortOrder sortOrder)
        {
            // sort the calls by date descending (newest to oldest)
            this.CallerIDType.Sort(new CallerIDTypeComparer(sortOrder));
        }

        /// <summary>
        /// The default file name for the call log
        /// </summary>
        internal static string FileName { get { return Path.Combine(Directory, Settings.Default.CallLogFile); } }

        /// <summary>
        /// The default directory where the call log lives
        /// </summary>
        internal static string Directory { get { return Application.CommonAppDataPath; } }

        /// <summary>
        /// Load the call log from the given string
        /// </summary>
        /// <param name="xmlString"></param>
        /// <returns></returns>
        internal static CallLog Deserialize(string xmlString)
        {
            return Serializer.Deserialize<CallLog>(xmlString);
        }

        /// <summary>
        /// Load a call log from the given file
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        internal static CallLog Load(string fileName)
        {
            return Serializer.DeserializeFromFile<CallLog>(fileName);
        }

        /// <summary>
        /// Load the default call log
        /// </summary>
        /// <returns></returns>
        internal static CallLog Load()
        {
            using (StreamReader reader = new StreamReader(CallLog.FileName))
            {
                StringBuilder sb = new StringBuilder(reader.ReadToEnd());
                sb.Insert(0, s_callLogHeader);
                sb.Append(s_callLogFooter);

                CallLog callLog = CallLog.Deserialize(sb.ToString());
                callLog.Month = DateTime.Now.Month.ToString();
                callLog.Year = DateTime.Now.Year.ToString();
                return callLog;
            }
        }

        internal void Save(string fileName)
        {
            Serializer.SerializeToFile<CallLog>(this, Path.Combine(Directory, fileName ));
        }

        /// <summary>
        /// Get a sorted list of the call logs in the call log directory
        /// </summary>
        /// <returns></returns>
        internal static StringCollection GetCallLogs()
        {
            List<string> fileList = new List<string>();
            string[] files = System.IO.Directory.GetFiles(CallLog.Directory, Resources.CallLogSearchPattern); //"CallLog??-????.xml"
            fileList.AddRange(files);

            if (fileList.Count > 1)
            {
                fileList.Sort();
                fileList.Reverse(1, fileList.Count - 1);
            }
            
            StringCollection fileCollection = new StringCollection();
            fileCollection.AddRange(fileList.ToArray());
            return fileCollection;
        }

    }

    /// <summary>
    /// Compare GataGridView rows and sort them by date
    /// </summary>
    class CallerIDTypeComparer : IComparer<CallerIDType>
    {
        private static int m_sortOrderModifier = 1;

        public CallerIDTypeComparer(SortOrder sortOrder)
        {
            if (sortOrder == SortOrder.Descending)
                m_sortOrderModifier = -1;
            else if (sortOrder == SortOrder.Ascending)
                m_sortOrderModifier = 1;
        }

        #region IComparer<CallerIDType> Members

        public int Compare(CallerIDType x, CallerIDType y)
        {
            // Try to sort based on the Last Name column.
            int compareResult = DateTime.Compare(x.Date, y.Date);

            return compareResult * m_sortOrderModifier;
        }

        #endregion
    }


    /// <summary>
    /// The callerID encodes and persistes callerID events
    /// </summary>
    public partial class CallerIDType
    {
        /// <summary>
        /// Default constructor
        /// </summary>
        public CallerIDType() { }

        /// <summary>
        /// Constructor for callerID that takes a name and number
        /// </summary>
        internal CallerIDType(string number, string name)
        {
            this.Name = name;
            this.Number = number;
            this.Date = DateTime.Now;
        }

        /// <summary>
        /// test if this callerID class has both a name and a number
        /// </summary>
        /// <returns></returns>
        internal bool IsComplete()
        {
            return (
                !string.IsNullOrEmpty(this.Name) &&
                !string.IsNullOrEmpty(this.Number));

        }

        /// <summary>
        /// Create a callerID for debug purposes
        /// </summary>
        /// <returns></returns>
        internal static CallerIDType CreateDebugCallerID()
        {
            return new CallerIDType("6045555555", "Test CallerID");
        }

        internal string Serialize()
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.OmitXmlDeclaration = true;

            return Serializer.Serialize<CallerIDType>(this, settings);
        }
    }

}
