//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.Collections.ObjectModel;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Threading;

using YAC_NET.Properties;
using YAC_NET.Data;

namespace YAC_NET.Data
{
    /// <summary>
    /// This is default configuration for this project.
    /// /// </summary>
    public partial class Configuration
    {
        /// <summary>
        /// This is default configuration for this project.
        /// </summary>
        public Configuration() 
        {
            this.NumberTranslation = new NumberTranslationTable();
            this.Port = 10629;
            Modem = new ModemConfigType();
            Listeners = new ListenerTypeCollection();
            this.Mode = ProgramMode.Server;
            this.Format = new FormatType();
            this.LogCalls = true;

            this.Version = Application.ProductVersion;
        }

        #region enumerations to data sources
        private static EnumCollection m_modeValues = new EnumCollection(typeof(ProgramMode));

        /// <summary>
        /// A program mode (Server or Listener)
        /// </summary>
        [XmlIgnore]
        public static EnumCollection ModeValue
        {
            get { return Configuration.m_modeValues; }
        }
        #endregion

        #region notifications
        /// <summary>
        /// Iterate through the listeners and send them the CallerID message
        /// </summary>
        /// <param name="callerID"></param>
        public void Notify(CallerIDType callerID)
        {
            foreach (ListenerType listener in this.Listeners)
                listener.Notify(callerID);
        }

        /// <summary>
        /// Iterate trought the listeners and send them a text message
        /// </summary>
        /// <param name="text"></param>
        public void Message(string text)
        {
            foreach (ListenerType listener in this.Listeners)
                listener.Message(text);
        }

        /// <summary>
        /// Iterate through the listeners and send them the RING message
        /// </summary>
        public void PhoneRing()
        {
            if (!this.ShowRing)
                return;
            foreach (ListenerType listener in this.Listeners)
                listener.Message("@RING\0");
        }
        #endregion

        #region static fields
        private static Configuration m_Configuration;
        private static string m_fileName;

        private bool m_isFirstRun = false;

        #endregion

        #region properties
        /// <summary>
        /// Indicates the program is being run for the first time
        /// </summary>
        [XmlIgnore]
        public bool IsFirstRun
        {
            get { return m_isFirstRun; }
            protected set { m_isFirstRun = value; }
        }

        /// <summary>
        /// Returns true if the program is in listener mode
        /// </summary>
        [XmlIgnore]
        public bool IsListener { get { return Mode == ProgramMode.Listener; } }

        /// <summary>
        /// Returns true if the program is in server mode
        /// </summary>
        [XmlIgnore]
        public bool IsServer { get { return Mode == ProgramMode.Server; } }

        /// <summary>
        /// The configuration file name
        /// </summary>
        [XmlIgnore]
        public static string FileName
        {
            get 
            { 
                if (string.IsNullOrEmpty(m_fileName))
                    m_fileName = Path.Combine(Application.CommonAppDataPath, Settings.Default.ConfigFile);
                return m_fileName;
            }
        }
        #endregion

        /// <summary>
        /// The current configuration
        /// </summary>
        public static Configuration Current 
        { 
            get 
            {
                if (m_Configuration == null)
                {
                    LoadConfig();
                }
                return m_Configuration; 
            } 
        }

        /// <summary>
        /// Migrte the old configuration to the current configuration
        /// This may involve reading an old file, or moving files from one directory to another.
        /// </summary>
        private static void MigrateOldConfigurations()
        {
            string dir = Directory.GetParent(Application.CommonAppDataPath).FullName;

            try
            {
                #region version 1.0.0.1
                string version1 = Path.Combine(dir, "1.0.0.1");
                if (Directory.Exists(version1))
                {
                    string configFile = Path.Combine(version1, Settings.Default.ConfigFile);
                    if (File.Exists(configFile))
                    {
                        Configuration config = Serializer.DeserializeFromFile<Configuration>(configFile);
                        Serializer.SerializeToFile<Configuration>(config, FileName);
                        File.Delete(configFile);

                        // move the log files
                        foreach (string file in Directory.GetFiles(version1))
                        {
                            string newName = Path.Combine(Application.CommonAppDataPath, Path.GetFileName(file));
                            File.Move(file, newName);
                        }
                    }

                    Directory.Delete(version1);
                }
                #endregion
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error reading old configuration:\r\n{0}", ex.ToString());
            }
        }

        /// <summary>
        /// Load the configuration from the file specified in the app.config
        /// </summary>
        public static void LoadConfig()
        {
            MigrateOldConfigurations();

            if (!File.Exists(FileName))
            {
                m_Configuration = new Configuration();
                m_Configuration.IsFirstRun = true;
                Serializer.SerializeToFile<Configuration>(m_Configuration, FileName);
            }
            else
            {
                m_Configuration = Serializer.DeserializeFromFile<Configuration>(FileName);
            }
        }

        /// <summary>
        /// Save the configuration from the file specified in app.config
        /// </summary>
        public void SaveConfig()
        {
            Serializer.SerializeToFile<Configuration>(m_Configuration, FileName);
        }
    }

    /// <summary>
    /// The numbertranslation table is a phone book that can translate 
    /// callerid-supplied phone numbers into friendly names that you record.
    /// for instance "Mom", "Dad, work"
    /// 
    /// This class is composed of the auto-generated NumberEntryTypeCollection which is the 
    /// list of name/number pairs contained in NumberEntryType. In this class we insert 
    /// these items into a dictionary for easy lookup. To maintain the dictionary across 
    /// changes to the underlying collection we handle the "Propertychanged" event.
    /// </summary>
    public partial class NumberTranslationTable
    {
        /// <summary>
        /// Default constructor
        /// </summary>
        public NumberTranslationTable() 
        {
            this.Table = new NumberEntryTypeCollection();
        }

        /// <summary>
        /// Translate a number from this callerID to a name in the phone book
        /// </summary>
        /// <param name="callerID"></param>
        internal void Translate(CallerIDType callerID)
        {
            foreach (NumberEntryType type in Table)
            {
                if (type.Number == callerID.Number)
                {
                    Trace.TraceInformation("Number found in phone book");
                    callerID.Name = type.Name;
                }
            }
        }

        /// <summary>
        /// Add a new value to the table 
        /// if the number is already in the table, then return false
        /// return true on success
        /// </summary>
        /// <param name="callerID"></param>
        /// <returns></returns>
        internal bool Add(CallerIDType callerID)
        {
            foreach (NumberEntryType type in Table)
                if (type.Number == callerID.Number)
                    return false;
            Table.Add(new NumberEntryType(callerID));
            return true;
        }
    }

    /// <summary>
    /// The ListenerType implements the information for the name and address of 
    /// a potential listener on the network to send callerID events to
    /// </summary>
    public partial class ListenerType
    {
        /// <summary>
        /// Default constructor
        /// </summary>
        public ListenerType() { }

        /// <summary>
        /// Send a CallerID message to a listener
        /// </summary>
        /// <param name="callerID"></param>
        public void Notify(CallerIDType callerID)
        {
            string message = string.Format("@CALL{0}~{1}\0", callerID.Name, callerID.Number);
            Byte[] data = Encoding.UTF8.GetBytes(message);
            Send(data);
        }

        /// <summary>
        ///  send a text message to a listener
        /// If you send the test "@RING\0", it means the phone is ringing
        /// </summary>
        /// <param name="text"></param>
        public void Message(string text)
        {
            string message = string.Format("{0}\0", text);
            Byte[] data = Encoding.UTF8.GetBytes(message);
            Send(data);
        }

        /// <summary>
        /// Method to send data to a listener
        /// </summary>
        /// <param name="data"></param>
        private void Send(byte[] data)
        {
            Thread thread = new Thread(SendThread);
            thread.Name = this.Name;
            thread.Start(data);
        }

        /// <summary>
        /// A thread to send the events over the network asynchronously
        /// </summary>
        /// <param name="obj"></param>
        private void SendThread(object obj)
        {
            try
            {
                byte[] data = obj as byte[];
                if (data == null)
                {
                    Trace.TraceError("Object passed to notifier send thread is not a byte array");
                    return;
                }

                // CallerID messages are formatted like this "@CALLBush George W.~(212) 555-9384\0"
                TcpClient client = new TcpClient(Address, (int)Configuration.Current.Port);

                NetworkStream stream = client.GetStream();

                // Send the message to the connected TcpServer. 
                stream.Write(data, 0, data.Length);
                stream.Close();
                client.Close();
            }
            catch (ArgumentNullException ex)
            {
                Trace.TraceError("ArgumentNullException:\r\n{0}", ex);
            }
            catch (SocketException ex)
            {
                Trace.TraceError("SocketException:\r\n{0}", ex);
            }

        }
    }

    /// <summary>
    /// The number entry represents a number in a phone book.
    /// Later we can expad this to contain photos and perhaps to lookup contact in Outlook
    /// </summary>
    public partial class NumberEntryType
    {
        /// <summary>
        /// default constructor
        /// </summary>
        public NumberEntryType() { }

        /// <summary>
        /// Contructor takes name/number pair
        /// </summary>
        /// <param name="name"></param>
        /// <param name="number"></param>
        public NumberEntryType(string name, string number)
        {
            Name = name;
            Number = number;
        }

        /// <summary>
        /// Contructor takes CallerIDType
        /// </summary>
        /// <param name="name"></param>
        /// <param name="number"></param>
        public NumberEntryType(CallerIDType callerID)
        {
            Name = callerID.Name;
            Number = callerID.Number;
        }
    }

    /// <summary>
    /// The FormatType defines some formatting rules for phone callerID related strings
    /// </summary>
    public partial class FormatType
    {
        /// <summary>
        /// Constructor, initializes defaults
        /// </summary>
        public FormatType()
        {
            this.DateTime = "d";
            this.Notification = "{0}\r\nCall from: {1}\r\n{2}";

            this.Number = new string[]
            {
                "#", 
                "##",
                "###",
                "####",
                "# ####",
                "## ####",
                "### ####",
                "# ### ####",
                "## ### ####",
                "(###) ### ####",
                "# (###) ### ####",
                "## (###) ### ####",
                "### (###) ### ####",
                "#### (###) ### ####",
                "##### (###) ### ####",
                "###### (###) ### ####",
                "####### (###) ### ####",
            };

        }

        /// <summary>
        /// Format the phone number for human reading
        /// use the above format strings to do this
        /// pre-requisite: the number is all Digits and contains no other characters
        /// </summary>
        /// <param name="number"></param>
        /// <returns></returns>
        public string FormatNumber(string number)
        {
            try
            {
                if (string.IsNullOrEmpty(number)) return string.Empty;

                // parse the number to strip all extra characters
                number = ParseNumber(number);
                
                if (number.Length <= this.Number.Length)
                {
                    long num = Convert.ToInt64(number);
                    return num.ToString(this.Number[number.Length - 1]);
                }
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error formatting a phone number: {0}\r\n{1}", number, ex.ToString());
            }

            return number;

        }

        /// <summary>
        /// Parse a phone number that has been formatted for display
        /// essentially strip all non-numeric characters
        /// </summary>
        /// <param name="number"></param>
        /// <returns></returns>
        public string ParseNumber(string number)
        {
            try
            {
                // empty strings are valid
                if (string.IsNullOrEmpty(number)) return string.Empty;

                // convert to a character array
                char[] charArray = number.ToCharArray();
                StringBuilder sb = new StringBuilder();

                // step through the array and check each character
                foreach (char ch in charArray)
                {
                    if (char.IsDigit(ch))
                        sb.Append(ch);
                }

                return sb.ToString();
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error parsing a phone number: {0}\r\n{1}", number, ex.ToString());
            }

            return number;
        }
    }
}
