//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Diagnostics;
using System.ServiceProcess;

using YAC_NET.Properties;
using YAC_NET.Data;

namespace YAC_NET.Forms
{
    /// <summary>
    /// The Main YAC.NET form
    /// You can use the entire program from the context menu strip
    /// the form gives you some additional help by showing help text
    /// when you over over the buttons.
    /// </summary>
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            myAppTitleLabel.Text = string.Format("{0} {1}", 
                Application.ProductName,
                Application.ProductVersion);
            myAppFootnoteLabel.Text = string.Format("{0} {1}  ",
                Application.ProductName,
                Application.ProductVersion);

            // hide some of the tool strip items when you're a service
            configureModemToolStripMenuItem.Visible = Configuration.Current.IsServer;
            listenersToolStripMenuItem.Visible = Configuration.Current.IsServer;
            runAsServiceToolStripMenuItem.Visible = Configuration.Current.IsServer;
            sendMessageToolStripMenuItem.Visible = Configuration.Current.IsServer;

            myRichTextBox.Rtf = Resources.RTFDefault;

            // if the service is installed, stop it before you open this form
            // this is so we can edit the config, and while we're doing this
            // we will handle all calls.
            try
            {
                UpdateServiceToolTipText();

                if (Utility.Utility.ServiceExists(Resources.ServiceName) && Configuration.Current.IsServer)
                {
                    myRunAsServiceCheckBox.Checked = true;
                    myServiceController.ServiceName = Resources.ServiceName;
                    myServiceController.Refresh();
                    if (myServiceController.Status == ServiceControllerStatus.Running)
                    {
                        myServiceController.Stop();
                        myServiceController.WaitForStatus(ServiceControllerStatus.Stopped, new TimeSpan(0, 0, 30));
                    }
                }
 
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error stopping service:\r\n{0}", ex.ToString());
                MessageBox.Show(
                    this,
                    string.Format(Resources.Error_StoppingService, ex.Message),
                    Resources.Caption_Error, 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }

        }

        #region fields
        IYAC m_yac;
        bool m_reallyClose = false;
        bool m_isHidden = true;
        bool m_newCalls = false;
        #endregion

        /// <summary>
        /// Load the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            InitYAC();

            if (Configuration.Current.IsFirstRun &&
                Configuration.Current.IsServer)
            {
                WelcomeForm.Show(this);
            }
            else
            {
                HideForm();
            }

            if (Configuration.Current.IsServer)
                myNotifyIcon.Text = string.Format(Resources.ServerStatus, Application.ProductName, Configuration.Current.Modem.Port);
            else
            {
                // by removing this panel all of the buttons that
                // are not valid are removed from the form
                // also, since the textbox expands, the form rearranges itself so
                // there's no blank space.
                myMainPanel.Controls.Remove(myPanel);
                myNotifyIcon.Text = string.Format(Resources.ListenerStatus, Application.ProductName);
                this.Text = string.Format(Resources.ListenerStatus, Application.ProductName);
            }
        }

        /// <summary>
        /// Handle a ring event
        /// </summary>
        void m_yac_PhoneRing()
        {
            if (this.InvokeRequired)
            {
                BeginInvoke((MethodInvoker)delegate()
                            {
                                HandleRing();
                            });
                return;
            }

            HandleRing();
        }

        /// <summary>
        /// Handle a caller ID event:
        /// </summary>
        /// <param name="callerID"></param>
        void m_yac_CallerID(CallerIDType callerID)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke((MethodInvoker)delegate()
                            {
                                HandleCallerID(callerID);
                            });
                return;
            }

            HandleCallerID(callerID);

        }

        /// <summary>
        /// Handle a message event
        /// </summary>
        /// <param name="message"></param>
        void m_yac_Message(string message)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke((MethodInvoker)delegate()
                            {
                                HandleMessage(message);
                            });
                return;
            }

            HandleMessage(message);

        }

        /// <summary>
        /// Exit the program, safe the configuration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myExitButton_Click(object sender, EventArgs e)
        {
            m_reallyClose = true;
            try
            {
                ShutdownYAC();
                Configuration.Current.SaveConfig();
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error closing YAC:\r\n{0}", ex.ToString());
            }

            Close();
        }

        /// <summary>
        /// Show the about box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox.Show(this);
        }

        /// <summary>
        /// Handle the closing event, don;t close unless the user has really 
        /// asked to do it via the menu or a button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // make sure we're not closing because windows is shutting down.
            if (!m_reallyClose && e.CloseReason != CloseReason.WindowsShutDown)
            {
                e.Cancel = true;
                HideForm();
            }
            else
            {
                try
                {
                    if (Utility.Utility.ServiceExists(Resources.ServiceName) && Configuration.Current.IsServer)
                    {
                        if (myServiceController.Status == ServiceControllerStatus.Stopped)
                        {
                            myServiceController.Start();
                            myServiceController.WaitForStatus(ServiceControllerStatus.Running, new TimeSpan(0,0,30));
                        }
                    }
                }
                catch (Exception ex)
                {
                    Trace.TraceError("Error starting service:\r\n{0}", ex.ToString());
                    MessageBox.Show(
                        this,
                        string.Format(Resources.Error_StoppingService, ex.Message),
                        Resources.Caption_Error,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// When the user double clicks on the notification icon, 
        /// show the main form.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myNotifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (m_newCalls)
            {
                myNotifyIcon.Icon = Resources.Phone;
                CallLogForm.Show(this);
                m_newCalls = false;
            }
            else
            {
                ShowForm();
            }
        }

        /// <summary>
        /// Save the configuration,
        /// restart YAC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myApplyButton_Click(object sender, EventArgs e)
        {
            ShutdownYAC();

            Configuration.Current.SaveConfig();

            // restart YAC
            InitYAC();
        }

        /// <summary>
        /// Revert to the saved configuration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myRevertButton_Click(object sender, EventArgs e)
        {
            Configuration.LoadConfig();
        }

        /// <summary>
        /// Don't show the form unless we really want to
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Shown(object sender, EventArgs e)
        {
            if (!m_isHidden)
                HideForm();
        }

        /// <summary>
        /// when the form is minimized, hide it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
                HideForm();
        }

        /// <summary>
        /// Configure the modem, 
        /// require shutting down YAC and restarting
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myConfigureModemButton_Click(object sender, EventArgs e)
        {
            ShutdownYAC();

            ConfigureModemForm.Show(this);

            InitYAC();

        }

        /// <summary>
        /// Send a text message to all YAC listeners
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mySendMessageButton_Click(object sender, EventArgs e)
        {
            SendMessageForm.Show(this);
        }

        /// <summary>
        /// Edit the list of listeners
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myListenersButton_Click(object sender, EventArgs e)
        {
            ListenerForm.Show(this);
        }

        /// <summary>
        /// Show and edit the call log
        /// When you've looked at the log, reset the icon to the default
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myCallLogButton_Click(object sender, EventArgs e)
        {
            m_newCalls = false;
            CallLogForm.Show(this);
            myNotifyIcon.Icon = Resources.Phone;
        }

        /// <summary>
        /// Show and edit the phone book
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myPhoneBookButton_Click(object sender, EventArgs e)
        {
            PhoneBookForm.Show(this);
        }

        /// <summary>
        /// Install the service (run YAC as a service)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myRunAsServiceCheckBox_Click(object sender, EventArgs e)
        {
            try
            {
                this.Cursor = Cursors.WaitCursor;
                if (Utility.Utility.ServiceExists(Resources.ServiceName))
                {
                    //myRunAsServiceCheckBox.Text = Resources.RunAsService;
                    Program.DoUninstall();
                }
                else
                {
                    myServiceController.ServiceName = Resources.ServiceName;
                    //myRunAsServiceCheckBox.Text = Resources.RunAsProgram;
                    Program.DoInstall();
                }

                UpdateServiceRTFText();

            }
            catch (Exception ex)
            {
                ExceptionManager.CriticalException(ex);
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        /// <summary>
        /// Close the form (hide)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myCloseButton_Click(object sender, EventArgs e)
        {
            HideForm();
        }

        /// <summary>
        /// Turn on or off call logging from the menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void logCallsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Configuration.Current.LogCalls = !Configuration.Current.LogCalls;
            logCallsToolStripMenuItem.Checked = Configuration.Current.LogCalls;
        }

        /// <summary>
        /// Send a debug caller ID message
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myCallerIDDebugButton_Click(object sender, EventArgs e)
        {
#if DEBUG
            m_yac.SendDebugCallerID();
#endif
        }

        /// <summary>
        /// Set some items on the main context menu.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            runAsServiceToolStripMenuItem.Checked = Utility.Utility.ServiceExists(Resources.ServiceName);
            logCallsToolStripMenuItem.Checked = Configuration.Current.LogCalls;
        }

        /// <summary>
        /// Configure YAC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myConfigureButton_Click(object sender, EventArgs e)
        {
            ConfigurationForm.Show(this);
        }

        /// <summary>
        /// If the user clicks the notify icon, close it
        /// and reset the notifyIcon
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myNotifyIcon_BalloonTipClicked(object sender, EventArgs e)
        {
            myNotifyIcon.Icon = Resources.Phone;
        }

        /// <summary>
        /// This is the mouse over event for the buttons on the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myButton_MouseEnter(object sender, EventArgs e)
        {
            Control control = sender as Control;
            if (control == null) return;

            control.Select();

            string tag = control.Tag as string;
            switch (tag)
            {
                case "ConfigureModem": myRichTextBox.Rtf = Resources.RTFConfigureModem; break;
                case "CallLog": myRichTextBox.Rtf = Resources.RTFCallLog; break;
                case "Configure": myRichTextBox.Rtf = Resources.RTFConfigure; break;
                case "Listeners": myRichTextBox.Rtf = Resources.RTFListeners; break;
                case "PhoneBook": myRichTextBox.Rtf = Resources.RTFPhoneBook; break;
                case "RunAsService": UpdateServiceRTFText(); break;
                case "SendMessage": myRichTextBox.Rtf = Resources.RTFSendMessage; break;
                case "Close": myRichTextBox.Rtf = Resources.RTFClose; break;
                case "Exit": myRichTextBox.Rtf = Resources.RTFExit; break;
                default: myRichTextBox.Rtf = Resources.RTFDefault; break;
            }
        }

        /// <summary>
        /// Update text related to the service check box,
        /// - tooltip text
        /// - RTF help text
        /// </summary>
        private void UpdateServiceRTFText()
        {
            if (Utility.Utility.ServiceExists(Resources.ServiceName))
            {
                this.myToolTip.SetToolTip(this.myRunAsServiceCheckBox, Resources.RunAsProgram);
                myRichTextBox.Rtf = Resources.RTFRunAsProgram;
            }
            else
            {
                this.myToolTip.SetToolTip(this.myRunAsServiceCheckBox, Resources.RunAsService);
                myRichTextBox.Rtf = Resources.RTFRunAsService;
            }
        }

        /// <summary>
        /// Update only the tooltip text for the service ckeckbox
        /// </summary>
        private void UpdateServiceToolTipText()
        {
            if (Utility.Utility.ServiceExists(Resources.ServiceName))
                this.myToolTip.SetToolTip(this.myRunAsServiceCheckBox, Resources.RunAsProgram);
            else
                this.myToolTip.SetToolTip(this.myRunAsServiceCheckBox, Resources.RunAsService);
        }

        /// <summary>
        /// Open the donation web page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myDonateButton_Click(object sender, EventArgs e)
        {
            Process.Start(Resources.URLDonate);
        }

        /// <summary>
        /// Go to the YAC.NET home page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mylinkLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://www.BlueToque.ca/Products/YAC.NET.html");
        }

        /// <summary>
        /// Show program help
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void helpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LaunchHelp();
        }

    }
}