//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;
using System.Diagnostics;
using System.Threading;

using YAC_NET.Properties;
using YAC_NET.Data;

namespace YAC_NET
{
    public delegate void CallerIDDelegate(CallerIDType callerId);
    public delegate void MessageDelegate(string message);
    public delegate void PhoneRingDelegate();

    /// <summary>
    /// The YAC class implements the IYAC interface
    /// it listens for events from the modem confugred in the default configuration
    /// </summary>
    class YAC : IYAC
    {
        #region fields
        SerialPort m_serialPort;
        StringBuilder m_buffer = new StringBuilder();
        CallerIDType m_callerID = new CallerIDType();
        #endregion

        #region events
        public event CallerIDDelegate CallerID;
        public event PhoneRingDelegate PhoneRing;
        public event MessageDelegate Message;
        #endregion

        /// <summary>
        /// constructor:
        /// configure the serial port and event handlers for it
        /// </summary>
        public YAC()
        {
            try
            {
                // Set up the serial port connection
                m_serialPort = new SerialPort(Configuration.Current.Modem.Port, 2400, Parity.None, 8, StopBits.One);
                m_serialPort.DtrEnable = true;
                m_serialPort.Handshake = Handshake.RequestToSend;
                m_serialPort.Encoding = Encoding.ASCII;

                // handle serial port callbacks
                m_serialPort.ErrorReceived += new SerialErrorReceivedEventHandler(m_serialPort_ErrorReceived);
                m_serialPort.DataReceived += new SerialDataReceivedEventHandler(m_serialPort_DataReceived);

            }
            catch (Exception ex)
            {
                Trace.TraceError("Error opening serial port {0}: \r\n{1}", m_serialPort.PortName, ex.ToString());
            }

        }


        #region IYAC
        /// <summary>
        /// Start listening for events from the modem
        /// </summary>
        public void Start()
        {
            Trace.TraceInformation("YAC CallerID starting");
            m_serialPort.Open();
            m_serialPort.Write(Configuration.Current.Modem.InitString+"\r");
        }
        
        /// <summary>
        /// Stop listening for events from the modem
        /// </summary>
        public void Stop()
        {
            Trace.TraceInformation("YAC CallerID stopping");
            m_serialPort.Close();
        }

        /// <summary>
        /// Send a simulared callerIDEvent
        /// </summary>
        public void SendDebugCallerID()
        {
            OnCallerID(CallerIDType.CreateDebugCallerID());
        }

        /// <summary>
        /// Send a simulated Ring event
        /// </summary>
        public void SendDebugRing()
        {
            OnRing();
        }

        /// <summary>
        /// Send a simulared message event
        /// </summary>
        public void SendDebugMessage()
        {
            OnMessage("Debug message");
        }

        #endregion
 
        #region event triggers
        /// <summary>
        /// Trigger a callerID event
        /// </summary>
        /// <param name="callerID"></param>
        protected void OnCallerID(CallerIDType callerID)
        {
            // set the date on this callerID
            callerID.Date = DateTime.Now;

            // conver the name to the local name if it's in the table
            Configuration.Current.NumberTranslation.Translate(callerID);
            
            // if someone's listening to this event, notify them
            if (CallerID != null)
                CallerID(callerID);

            m_callerID = new CallerIDType();
        }

        /// <summary>
        /// trigger a phone ringing event
        /// </summary>
        protected void OnRing()
        {
            // if someone wants to know that the phone is ringin, tell them
            if (PhoneRing != null)
                PhoneRing();

            // create a new callerID object on a ring (initialize callerID)
            m_callerID = new CallerIDType();
        }

        /// <summary>
        /// trigger a message event
        /// </summary>
        /// <param name="message"></param>
        protected void OnMessage(string message)
        {
            // if someone wants to know that the phone is ringin, tell them
            if (Message != null)
                Message(message);
        }
        #endregion

        #region serial port event handlers
        /// <summary>
        /// When an error happens while receiving data on the serial port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void m_serialPort_ErrorReceived(object sender, SerialErrorReceivedEventArgs e)
        {
            Trace.TraceError("Error on serial port {0}: {1}", m_serialPort.PortName, e.EventType.ToString());
        }

        /// <summary>
        /// Called when data is received on the serial port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void m_serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // read the data and write the modem log
            string value = m_serialPort.ReadExisting();
            Trace.Write(value, "ModemLog");

            // gather data in a buffer until the carriage return
            string[] data = value.Split(new char[] { '\n' }, StringSplitOptions.RemoveEmptyEntries);

            #region check if this data matched data in the buffer
            if (data.Length == 0)
                return;
            if (m_buffer.Length != 0)
            {
                m_buffer.Append(data[0]);
                data[0] = m_buffer.ToString();
                m_buffer = new StringBuilder();
            }
            #endregion

            foreach (string val in data)
            {
                #region check if all the data has arrived
                // if there's no '\r' then not all of the data has arrived
                if (!val.EndsWith("\r"))
                {
                    m_buffer.Append(val);
                    continue;
                }
                #endregion

                string ev = val.Trim(new char[] { '\r','\0' }).ToUpper();
                if (string.IsNullOrEmpty(ev)) continue;

                // split the data into parts
                string[] parts = ev.Split('=');

                switch (parts[0].Trim())
                {
                    case "RING":
                        OnRing();
                        break;

                    case "DATE": break;
                    case "TIME": break;

                    case "NMBR":
                        if (parts.Length == 2)
                            m_callerID.Number = parts[1].Trim();
                        break;

                    case "NAME":
                        if (parts.Length == 2)
                            m_callerID.Name = parts[1].Trim();
                        break;
                }

                if (m_callerID.IsComplete())
                    OnCallerID(m_callerID);
            }
        }
        #endregion

    }
}
