//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.ServiceProcess;
using System.Diagnostics;
using System.IO;

using YAC_NET.Properties;
using YAC_NET.Data;

namespace YAC_NET.Forms
{
    public partial class MainForm
    {

        /// <summary>
        /// Initialze YAC
        /// </summary>
        private void InitYAC()
        {
            // if this is the first run of the program, and it is the server,
            // return. We can't start listening until he modem is configured
            if (Configuration.Current.IsFirstRun && 
                Configuration.Current.IsServer)
                return;

            // if the port is not set, then we're starting the program for the first time.
            // We exit immediately before initializing the YAC class and m_firstStartup 
            // stays true
            if (string.IsNullOrEmpty(Configuration.Current.Modem.Port) &&
                Configuration.Current.IsServer)
            {
                MessageBox.Show(
                    this,
                    Resources.Error_ModemPortNotSet,
                    Resources.Error_ModemPortNotSetTitle,
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
                return;
            }

            try
            {
                m_yac = CreateYAC();
                m_yac.CallerID += new CallerIDDelegate(m_yac_CallerID);
                m_yac.PhoneRing += new PhoneRingDelegate(m_yac_PhoneRing);
                m_yac.Message += new MessageDelegate(m_yac_Message);
                m_yac.Start();
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error creating IYAC:\r\n{0}", ex.ToString());
            }
        }

        /// <summary>
        /// Shutdown YAC
        /// </summary>
        private void ShutdownYAC()
        {
            if (m_yac != null)
                m_yac.Stop();
        }

        /// <summary>
        /// Create a new instance if IYAC depending on weather this program is being run as a 
        /// server or a listener
        /// </summary>
        /// <returns></returns>
        private IYAC CreateYAC()
        {
            if (Configuration.Current.IsListener)
                return new Listener();
            else
                return new YAC();
        }

        /// <summary>
        /// Show this form
        /// </summary>
        private void ShowForm()
        {
            m_isHidden = false;
            this.WindowState = FormWindowState.Normal;
            this.ShowInTaskbar = true;
            this.Visible = true;
            this.Show();
            this.BringToFront();
        }

        /// <summary>
        /// Hide this form
        /// </summary>
        private void HideForm()
        {
            this.WindowState = FormWindowState.Minimized;
            this.ShowInTaskbar = false;
            m_isHidden = true;
            this.Hide();
            this.Visible = false;
        }

        /// <summary>
        /// Handle the caller ID Event
        /// - show it to the user
        /// - send it to the listeners
        /// - optionally log it to a file
        /// </summary>
        /// <param name="callerId"></param>
        private void HandleCallerID(CallerIDType callerID)
        {
            try
            {
                // change the call notification icon to let the user know we've had new calls
                myNotifyIcon.Icon = Resources.Phone_red;
                m_newCalls = true;

                myNotifyIcon.ShowBalloonTip(
                    2000,
                    Resources.PhoneRinging,
                    string.Format(
                        Configuration.Current.Format.Notification, //"{0}\r\nCall from: {1}\r\n{2}", //Resources.CallFormat,
                        callerID.Date.ToString(Configuration.Current.Format.DateTime),
                        callerID.Name,
                        Configuration.Current.Format.FormatNumber(callerID.Number)),
                    ToolTipIcon.Info
                );

                // notify Listeners
                if (Configuration.Current.IsServer)
                    Configuration.Current.Notify(callerID);

                // if we're logging calls, send them to the log
                if (Configuration.Current.LogCalls)
                    CallLogger.LogCall(callerID);
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error handling CallerID event:\r\n{0}", ex.ToString());
            }
        }

        /// <summary>
        /// Handle a ring event
        /// - optionally change the icon to indicat that the phone is ringing
        /// </summary>
        private void HandleRing()
        {
            try
            {
                if (!Configuration.Current.ShowRing)
                    return;

                if (Configuration.Current.IsServer)
                    Configuration.Current.PhoneRing();

                #region show the phone is ringing
                Thread showRingThread = new Thread(new ThreadStart(delegate()
                {
                    try
                    {
                        myNotifyIcon.Icon = Resources.Phone_red;
                        Thread.Sleep(1000);
                        myNotifyIcon.Icon = Resources.Phone;
                    }
                    catch (Exception ex)
                    {
                        Trace.TraceError("Error displaying ring notification: {0}", ex.ToString());
                    }
                }));

                showRingThread.Name = "Show Ring Thread";
                showRingThread.Start();
                #endregion
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error handling Ring event:\r\n{0}" , ex.ToString());
            }
        }

        /// <summary>
        /// Handle the text message
        /// </summary>
        /// <param name="message"></param>
        private void HandleMessage(string message)
        {
            try
            {
                myNotifyIcon.ShowBalloonTip(
                    2000,
                    Resources.MessageTitle,
                    string.Format(message),
                    ToolTipIcon.None
                );
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error handling Message event:\r\n{0}" , ex.ToString());
            }

        }

        /// <summary>
        /// Launch help web page
        /// </summary>
        private static void LaunchHelp()
        {
            string url = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "Help\\index.htm");
            Process.Start(url);
        }


    }
}
