//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Diagnostics;
using System.Net;

using YAC_NET.Data;

namespace YAC_NET
{
    /// <summary>
    /// The listener class listens for CallerID events and message events on the
    /// network. It opens the port configures in the default configuration.
    /// It listens for connections on the network and decodes the text received, and
    /// triggers the appropriate evetns.
    /// </summary>
    class Listener : IYAC
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public Listener() { }

        #region events
        /// <summary>
        /// CallerID event triggered when a callerID events is received from the network
        /// </summary>
        public event CallerIDDelegate CallerID;

        /// <summary>
        /// PhoneRing event is triggered when a phone ring event is received from the network
        /// </summary>
        public event PhoneRingDelegate PhoneRing;

        /// <summary>
        /// The message event is triggered when a message is received from the network
        /// </summary>
        public event MessageDelegate Message;
        #endregion

        #region fields
        private TcpListener m_listener;
        private bool m_isClosing = false;
        #endregion

        #region event triggers
        protected void OnCallerID(CallerIDType callerID)
        {
            // translate messages received by the listener
            Configuration.Current.NumberTranslation.Translate(callerID);

            // if someone's listening to this event, notify them
            if (CallerID != null)
                CallerID(callerID);
        }
 
        protected void OnMessage(string message)
        {
            // if someone's listening to this event, notify them
            if (Message != null)
                Message(message);
        }

        protected void OnRing()
        {
            // if someone wants to know that the phone is ringin, tell them
            if (PhoneRing != null)
                PhoneRing();
        }
        #endregion

        #region IYAC
        /// <summary>
        /// Start the listener (implements
        /// </summary>
        public void Start()
        {
            // we do not care which IP Adderss we're listening on
            m_listener = new TcpListener(IPAddress.Any, (int)Configuration.Current.Port);
            m_listener.Start();
            m_listener.BeginAcceptTcpClient(new AsyncCallback(DoAcceptClientCallback), m_listener);
        }

        /// <summary>
        /// Stop the listener
        /// </summary>
        public void Stop()
        {
            m_isClosing = true;
            m_listener.Stop();
        }

        #region debug methods
#if DEBUG
        /// <summary>
        /// Send a fake callerID event
        /// </summary>
        public void SendDebugCallerID()
        {
            CallerIDType type = new CallerIDType();
            type.Name = "Test CallerID";
            type.Number = "6041111111";
            type.Date = DateTime.Now;
            OnCallerID(type);
        }

        /// <summary>
        /// Send a simulated Ring event
        /// </summary>
        public void SendDebugRing()
        {
            OnRing();
        }

        /// <summary>
        /// Send a simulared message event
        /// </summary>
        public void SendDebugMessage()
        {
            OnMessage("Debug message");
        }
#endif
        #endregion
        
        #endregion

        #region private members
        /// <summary>
        /// Process the client connection
        /// </summary>
        /// <param name="ar"></param>
        private void DoAcceptClientCallback(IAsyncResult ar)
        {
            try
            {
                // Get the listener that handles the client request.
                TcpListener listener = (TcpListener)ar.AsyncState;

                if (m_isClosing)
                    return;

                // End the operation and display the received data on the console.
                TcpClient client = listener.EndAcceptTcpClient(ar);

                m_listener.BeginAcceptTcpClient(new AsyncCallback(DoAcceptClientCallback), m_listener);

                Trace.TraceInformation("Connected!");

                Byte[] bytes = new Byte[256];
                String data = null;

                // Get a stream object for reading and writing
                NetworkStream stream = client.GetStream();

                int i;

                // Loop to receive all the data sent by the client.
                while ((i = stream.Read(bytes, 0, bytes.Length)) != 0)
                {
                    // Translate data bytes to a ASCII string.
                    data = System.Text.Encoding.UTF8.GetString(bytes, 0, i);
                    Trace.TraceInformation("Received: {0}", data);

                }

                // Shutdown and end connection
                client.Close();

                // Process the connection here. (Add the client to a server table, read data, etc.)
                Trace.TraceInformation("Client connected completed");

                if (data.StartsWith("@CALL"))
                {
                    string[] parameters = data.Split(new string[] { "@CALL", "~", "\0" }, StringSplitOptions.RemoveEmptyEntries);
                    if (parameters.Length != 2)
                    {
                        Trace.TraceWarning("Badly formatted @CALL string: {0}", data);
                        return;
                    }
                    CallerIDType callerID = new CallerIDType();
                    callerID.Name = parameters[0];
                    callerID.Number = parameters[1];
                    callerID.Date = DateTime.Now;
                    OnCallerID(callerID);
                }
                else if (data.StartsWith("@RING"))
                {
                    OnRing();
                }
                else
                {
                    OnMessage(data.TrimEnd('\0'));
                }

            }
            catch (Exception ex)
            {
                Trace.TraceError("Error receiving message over TCPIP on port {0}:\r\n{1}", Configuration.Current.Port, ex.ToString());
            }
        }
        #endregion

    }
}
