//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Windows.Forms;
using System.IO;

using YAC_NET.Properties;
using YAC_NET.Forms;
using YAC_NET.Data;
using YAC_NET.Install;

namespace YAC_NET
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static int Main(string[] argv)
        {
            int returnValue = 0;
            try
            {
                AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
                Configuration.LoadConfig();

                #region command line processing
                string param = argv.Length == 0 ? null : argv[0].ToLower().Replace('-', '/');
                switch (param)
                {
                    case "/i": returnValue = DoInstall(); break;

                    case "/u": returnValue = DoUninstall(); break;

                    case "/s": returnValue = RunService(); break;

                    case "/?": DoHelp(); break;

                    case "/l": Configuration.Current.Mode = ProgramMode.Listener; RunProgram(); break;

                    default: RunProgram(); break;
                }
                #endregion

            }
            catch (Exception ex)
            {
                ExceptionManager.CriticalException(ex);
            }

            return returnValue;

        }

        /// <summary>
        /// Run the program in server mode (not service).
        /// In server mode the program detects the caller id, 
        /// interacts with the local desktop and optionally sends
        /// callerID events to any listeners. There can only be
        /// server active on a machine at any time.
        /// </summary>
        private static void RunProgram()
        {
            // if the program is being run as a listener, one person could have it running
            // in each terminal of a termain services. Otherwise, if it's in server mode
            // then only one persone can have it running per machine.
            Scope scope = (Configuration.Current.IsListener) ? Scope.Session : Scope.Global;

            if (!SingleInstance.IsSingleInstance(scope, Application.ProductName))
            {
                MessageBox.Show(
                    Resources.Message_SingleInstance, 
                    Resources.Caption_Error, 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
                return;
            }

            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new MainForm());
        }

        /// <summary>
        /// Show the user help
        /// </summary>
        private static void DoHelp()
        {
            Console.WriteLine("{0} Command line arguments:", Resources.ServiceName);
            Console.WriteLine("\t{0} [/i] [/u] [/d] [/l] [/?]", Path.GetFileNameWithoutExtension(Application.ExecutablePath));
            Console.WriteLine("\t/i : Install as a service");
            Console.WriteLine("\t/u : Uninstall the service");
            Console.WriteLine("\t/l : Run as a listener");
            Console.WriteLine("\t/? : Show this help");
        }

        /// <summary>
        /// Run the program in service mode
        /// In service mode the program is installed as a windows NT service.
        /// It listens for CallerID events and sends them to listeners.
        /// There can only be one instance in service mode, but this is 
        /// handled by windows service controler.
        /// </summary>
        /// <returns></returns>
        private static int RunService()
        {
            try
            {
                YacService myService = new YacService();
                System.ServiceProcess.ServiceBase.Run(myService);
            }
            catch (Exception ex)
            {
                ApplicationException appEx = new ApplicationException(Resources.ServiceError, ex);
                if (ExceptionManager.CriticalException(appEx))
                    return -1;
            }
            return 0;
        }

        /// <summary>
        /// Install the service
        /// returs 0 for success, -1 for failure
        /// </summary>
        /// <returns></returns>
        internal static int DoUninstall()
        {
            try
            {
                LogTraceManager.LogCritical(Resources.Message_Uninstalling);

                YacServiceInstaller.DoUninstall(
                    Resources.ServiceName,
                    Resources.ServiceName,
                    Resources.ServiceDescription);

                LogTraceManager.LogCritical(Resources.Message_Uninstalled);
            }
            catch (Exception ex)
            {
                ApplicationException appEx = new ApplicationException(Resources.Error_InstallUninstall, ex);
                if (ExceptionManager.CriticalException(appEx))
                    return -1;
            }
            return 0;

        }

        /// <summary>
        /// Uninstall the service
        /// returs 0 for success, -1 for failure
        /// </summary>
        /// <returns></returns>
        internal static int DoInstall()
        {
            try
            {
                LogTraceManager.LogCritical(Resources.Message_Installing);

                YacServiceInstaller.DoInstall(
                    Resources.ServiceName,
                    Resources.ServiceName,
                    Resources.ServiceDescription);

                LogTraceManager.LogCritical(Resources.Message_Installed);
            }
            catch (Exception ex)
            {
                ApplicationException appEx = new ApplicationException(Resources.Error_InstallUninstall, ex);
                if (ExceptionManager.CriticalException(appEx))
                    return -1;
            }
            return 0;

        }

        /// <summary>
        /// What to do if there is an unhadled exception on a thread.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            LogTraceManager.LogCritical(e.Exception.ToString());
        }

        /// <summary>
        /// What to do if there is an exception and we have not handled it elsewhere.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            LogTraceManager.LogCritical(string.Format(Resources.Message_UnhandledException, e.ExceptionObject.ToString()));
        }
    }
}